/*
    Neighand: Neighborhood Handling library

    The goal of this project is to find 3D neighbors efficiently.
    Read the documentation contained in the article "Spherical
    Indexing for Neighborhood Queries" for full details.

    This file defines the functions for applying a user-defined
    functor or callback to all objects in the region of interest.

    Nicolas Brodu, 2006
    Code released according to the GNU GPL, v2 or above.
*/

#if defined(NEIGHAND_C_CALLBACK_API)

#if defined(NEIGHAND_SPECIALIZE)
template<> void NeighborhoodHandler<NEIGHAND_SPECIALIZE>::applyToAll(Callback f, void* userData)
#else
template <int exp2divx, int exp2divy, int exp2divz, bool wrapX, bool wrapY, bool wrapZ, bool layerZ>
NEIGHAND_INLINE void NeighborhoodHandler<NEIGHAND_TEMPLATE_ARGUMENTS>::applyToAll(Callback f, void* userData)
#endif

#else

#if defined(NEIGHAND_SPECIALIZE)
#ifdef NEIGHAND_SPECIALIZE_FUNCTOR
template<> template<> NEIGHAND_SPECIALIZE_FUNCTOR NeighborhoodHandler<NEIGHAND_SPECIALIZE>::applyToAll<NEIGHAND_SPECIALIZE_FUNCTOR>(NEIGHAND_SPECIALIZE_FUNCTOR f)
#else
template<> template<> CallbackToFunctor NeighborhoodHandler<NEIGHAND_SPECIALIZE>::applyToAll<CallbackToFunctor>(CallbackToFunctor f)
#endif
#else
template <int exp2divx, int exp2divy, int exp2divz, bool wrapX, bool wrapY, bool wrapZ, bool layerZ>
template<typename Functor>
NEIGHAND_INLINE Functor NeighborhoodHandler<NEIGHAND_TEMPLATE_ARGUMENTS>::applyToAll(Functor f)
#endif

#endif

{
    ObjectProxy** cList = cachedLists;
    ObjectProxy** lastList = cachedLists + WrapHelper<NEIGHAND_TEMPLATE_ARGUMENTS>::ArraySize;
    while (cList!=lastList) {
        ObjectProxy* plist = *cList++;
        #define NEIGHAND_APPLY_CHECK_FIRST
        // define this flag so there is no object check
        #define NEIGHAND_APPLY_XYZ_API 1
        #include "neighand_apply_processlines_uncond.hpp"
        #undef NEIGHAND_APPLY_XYZ_API
        #undef NEIGHAND_APPLY_CHECK_FIRST
    }

#ifndef NEIGHAND_C_CALLBACK_API
    return f;
#endif
}
