#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;

#include <math.h>

#include "neighand.h"
using namespace neighand;


struct Agent {
    int number;
    FloatType x,y,z; // to find back the position for LQ
};

// Declare how many agents to put in the DB
const int Nagents = 10000;
Agent* agents;
vector<int> nhneighbors, sqneighbors;
FloatType sizeX, sizeY, sizeZ;

template <bool wrapX, bool wrapY, bool wrapZ>
inline FloatType squaredDistance(FloatType dx,FloatType dy,FloatType dz);

template <>
inline FloatType squaredDistance<false,false,false>(FloatType dx,FloatType dy,FloatType dz)
{
    return dx*dx+dy*dy+dz*dz;
}

template <>
inline FloatType squaredDistance<true,true,false>(FloatType dx,FloatType dy,FloatType dz)
{
    dx = remainderf(dx, sizeX);
    dy = remainderf(dy, sizeY);
    return dx*dx+dy*dy+dz*dz;
}

template <>
inline FloatType squaredDistance<true,true,true>(FloatType dx,FloatType dy,FloatType dz)
{
    dx = remainderf(dx, sizeX);
    dy = remainderf(dy, sizeY);
    dz = remainderf(dz, sizeZ);
    return dx*dx+dy*dy+dz*dz;
}

struct VectorFunctor {
    vector<int>* nbagents;
    VectorFunctor(vector<int>* nb) : nbagents(nb) {}
    void operator()(void * object) {
        Agent* agent = reinterpret_cast<Agent*>(object);
        nbagents->push_back(agent->number);
    }
};

// VISUAL INSPECTION DEBUG CODE
void showLists(FloatType x,FloatType y,FloatType z,FloatType d) {

    cout << "dsq="<<(d*d)<<", center=("<<x<<", "<<y<<", "<<z<<")"<<endl;

    cout << "NH\tSQ" << endl;
    for (uint32_t a=0; a<min(nhneighbors.size(), sqneighbors.size()); ++a) {
        cout << nhneighbors[a] << "\t" << sqneighbors[a] << "\t" << "nh: dsq=" <<
        squaredDistance<CONSISTENCY_TEST_WRAP>(x-agents[nhneighbors[a]].x, y-agents[nhneighbors[a]].y, z-agents[nhneighbors[a]].z) << " at (" << agents[nhneighbors[a]].x << ", " << agents[nhneighbors[a]].y<<", " <<agents[nhneighbors[a]].z << "), sq: d^2=" <<
        squaredDistance<CONSISTENCY_TEST_WRAP>(x-agents[sqneighbors[a]].x, y-agents[sqneighbors[a]].y, z-agents[sqneighbors[a]].z) << " at (" << agents[sqneighbors[a]].x << ", " << agents[sqneighbors[a]].y<<", " <<agents[sqneighbors[a]].z << ")"
        <<endl;
    }
    if (nhneighbors.size()>sqneighbors.size()) for (uint32_t a=sqneighbors.size(); a<nhneighbors.size(); ++a) {
        cout << nhneighbors[a] << "\t    \t" << "nh: dsq=" <<
        squaredDistance<CONSISTENCY_TEST_WRAP>(x-agents[nhneighbors[a]].x, y-agents[nhneighbors[a]].y, z-agents[nhneighbors[a]].z) << " at (" << agents[nhneighbors[a]].x << ", " << agents[nhneighbors[a]].y<<", " <<agents[nhneighbors[a]].z << ")"
        <<endl;
    }
    if (sqneighbors.size()>nhneighbors.size()) for (uint32_t a=nhneighbors.size(); a<sqneighbors.size(); ++a) {
        cout << "    \t" << sqneighbors[a] << "\t" << "sq: dsq=" <<
        squaredDistance<CONSISTENCY_TEST_WRAP>(x-agents[sqneighbors[a]].x, y-agents[sqneighbors[a]].y, z-agents[sqneighbors[a]].z) << " at (" << agents[sqneighbors[a]].x << ", " << agents[sqneighbors[a]].y<<", " <<agents[sqneighbors[a]].z << ")"
        <<endl;
    }
}

int main() {


    // 32x32x32 bloc, from 10 to 54.8 (random values)
    typedef NeighborhoodHandler<5,5,5,CONSISTENCY_TEST_WRAP> NH;
    NH nh(10.f, 10.f, 10.f, 1.4f, "consistency.bin");
    sizeX = 44.8;
    sizeY = 44.8;
    sizeZ = 44.8;

    agents = new Agent[Nagents];
    ObjectProxy** nhproxies = new ObjectProxy*[Nagents];

    // Put them in
    for (int i=0; i<Nagents; ++i) {
        agents[i].number = i;
        agents[i].x = 64.8 * (rand() / (RAND_MAX + 1.0f));
        agents[i].y = 64.8 * (rand() / (RAND_MAX + 1.0f));
        agents[i].z = 64.8 * (rand() / (RAND_MAX + 1.0f));

        // Insert in NH
        nhproxies[i] = nh.insert(agents[i].x, agents[i].y, agents[i].z, &agents[i]);
    }

    int Nqueries = 100000;
    int NmovesPerQuery = 500;
    bool globalFailed = false;

    // Now call the callbackFunction on random queries with both, and compare
    // Use stl for sorting algorithms
    for (int i=0; i<Nqueries; ++i) {

        // First randomize the positions.
        // This tests that the move operation works as intended
        for (int j=0; j<NmovesPerQuery; ++j) {
            int a = rand() % Nagents;
            agents[a].x = 64.8 * (rand() / (RAND_MAX + 1.0f));
            agents[a].y = 64.8 * (rand() / (RAND_MAX + 1.0f));
            agents[a].z = 64.8 * (rand() / (RAND_MAX + 1.0f));
            nh.update(agents[a].x, agents[a].y, agents[a].z, nhproxies[a]);
        }

        // Now perform a query
        // choose a distance that covers all case (completely out, intersect, englobes all)
        FloatType distance = 64.8f * (rand() / (RAND_MAX + 1.0f));
        FloatType distSquared = distance * distance;
        // choose point anywhere, possibly out
        FloatType x = 64.8f * (rand() / (RAND_MAX + 1.0f));
        FloatType y = 64.8f * (rand() / (RAND_MAX + 1.0f));
        FloatType z = 64.8f * (rand() / (RAND_MAX + 1.0f));
        // Pass a build list as argument
        nhneighbors.clear(); sqneighbors.clear();

        ClosestNeighbor closest;
        ClosestNeighbor knn[20]; int knnfound;

        int closestSelected = rand()%3;
        switch (closestSelected) {
            // find all neighbors
            case 0:
                nh.applyToNeighbors(x, y, z, distance, VectorFunctor(&nhneighbors));
                // simple query algo
                for (int a = 0; a<Nagents; ++a) {
                    if (squaredDistance<CONSISTENCY_TEST_WRAP>(agents[a].x-x, agents[a].y-y, agents[a].z-z)<=distSquared) {
                        sqneighbors.push_back(a);
                    }
                }
                break;
            // Nearest neighbor only.
            case 1:
                nh.findClosestNeighbor(x, y, z, distance, &closest);
                if (closest.object!=0) nhneighbors.push_back(reinterpret_cast<Agent*>(closest.object)->number);
                { FloatType dmin = distSquared; int amin = -1;
                // simple query algo
                for (int a = 0; a<Nagents; ++a) {
                    FloatType dsq=squaredDistance<CONSISTENCY_TEST_WRAP>(agents[a].x-x, agents[a].y-y, agents[a].z-z);
                    if (dsq <= dmin) {
                        dmin = dsq;
                        amin = a;
                    }
                }
                if (amin!=-1) sqneighbors.push_back(amin);
                }
                break;
            // find the K nearest neighbors, reasonable random K range
            case 2: {
                int N = 1+(rand()%20);
                knnfound = nh.findClosestNeighbors(x, y, z, distance, knn, N);
                for (int i=0; i<knnfound; ++i) nhneighbors.push_back(reinterpret_cast<Agent*>(knn[i].object)->number);
                // sq now
                {
                    for (int i=0; i<N; ++i) {
                        *reinterpret_cast<uint32_t*>(&knn[i].squaredDistance) = uint32_t(0x7F7FFFFF);
                    }
                    int nfound = 0;
                    for (int a = 0; a<Nagents; ++a) {
                        FloatType dsq=squaredDistance<CONSISTENCY_TEST_WRAP>(agents[a].x-x, agents[a].y-y, agents[a].z-z);
                        if (dsq <= distSquared) {
                            if (++nfound>N) nfound=N;
                            ClosestNeighbor currentObject;
                            currentObject.squaredDistance = dsq;
                            currentObject.object = &agents[a];
                            // keep only the min. don't reduce search distance here
                            for (int i=0; i<nfound; ++i) if (dsq < knn[i].squaredDistance) {
                                ClosestNeighbor tmp = knn[i];
                                knn[i] = currentObject;
                                currentObject = tmp;
                            }
                        }
                    }
                    for (int i=0; i<nfound; ++i) sqneighbors.push_back(reinterpret_cast<Agent*>(knn[i].object)->number);
                }
                break;
            }
        }


//        showLists(x,y,z,distance);

        // post-process: sort the integers for comparison
        sort(nhneighbors.begin(), nhneighbors.end());
        sort(sqneighbors.begin(), sqneighbors.end());

        bool failed = false;

        // AUTOMATED TEST
        if (nhneighbors.size() != sqneighbors.size()) {
            failed = true;
        }
        for (uint32_t a=0; a<nhneighbors.size(); ++a) {
            if (nhneighbors[a] != sqneighbors[a]) {
            failed = true;
            }
        }

        if (failed) {
            cout << "FAILED " << (closestSelected?((closestSelected==1)?"(find closest)":"(find K nearest)"):"(find all)") << endl;
            showLists(x,y,z,distance);
            globalFailed = true;
        }
    }

    if (globalFailed) {
        cout << endl;
        cout << "Note: Consistency test may fail for some configurations due to floating points limited precision." << endl;
        cout << "Please check the details above if this could explain the mismatch (compare the dsq values that are displayed)." << endl;
    }
    else cout << Nqueries << " random tests OK" << endl;

}


